# -*- coding: utf-8 -*-

"""Functions for generating a project from a project template."""
from __future__ import unicode_literals

from abc import ABCMeta, abstractmethod
from os import listdir
from os.path import dirname, basename

__all__ = [
    basename(f)[:-3]
    for f in listdir(dirname(__file__))
    if f[-3:] == ".py" and not f.endswith("__init__.py")
]


class BaseOperator(metaclass=ABCMeta):
    """Base hook mixin class."""

    def __init__(self, operator_dict, context=None):
        """Initialize Basehook."""
        self.operator_dict = operator_dict
        self.context = context or {}

    @abstractmethod
    def execute(self):
        pass



# -*- coding: utf-8 -*-

"""Functions for discovering and executing various cookiecutter operators."""
from cookiecutter.environment import StrictEnvironment, render_variable
from cookiecutter.operators import *  # noqa

from cookiecutter.operators import BaseOperator


def run_operator(operator_dict: dict, no_input: bool = False) -> list:
    """Run operator."""
    operator_output = None
    operator_list = BaseOperator.__subclasses__()
    for o in operator_list:
        if operator_dict['type'] == o.type:  # noqa
            operator = o(operator_dict)
            # TODO: Determine if default method is needed across all operators
            # If so, need to feed in `no_input` from both cases in call
            operator_output = operator.execute()

    return operator_output


def parse_operator(
    context, key, cookiecutter_dict, append_key: bool = False, no_input: bool = False,
):
    """Parse input dict for loop and when logic and calls hooks.

    :return: cookiecutter_dict # noqa
    """
    env = StrictEnvironment(context=context)
    operator_dict = context['cookiecutter'][key]

    # Extract loop
    if 'loop' in operator_dict:
        loop_targets = render_variable(env, operator_dict['loop'], cookiecutter_dict)
        operator_dict.pop('loop')

        loop_output = []
        for l in loop_targets:
            loop_cookiecutter = cookiecutter_dict
            loop_cookiecutter.update({'item': l})
            loop_output += [
                parse_operator(
                    context, key, loop_cookiecutter, append_key=True, no_input=no_input
                )
            ]

        cookiecutter_dict.pop('item')
        cookiecutter_dict[key] = loop_output
        return cookiecutter_dict

    if 'when' in operator_dict:
        if not context:
            raise ValueError("Can't have when condition without establishing context")

        when_condition = (
            render_variable(env, operator_dict['when'], cookiecutter_dict) == 'True'
        )

        operator_dict.pop('when')
        if not isinstance(when_condition, bool):
            raise ValueError("When condition needs to render with jinja to boolean")

    else:
        when_condition = True

    if when_condition:
        operator_dict = render_variable(env, operator_dict, cookiecutter_dict)
        if not no_input:
            # Run prompt
            cookiecutter_dict[key] = run_operator(operator_dict)  # output is list
        elif 'default' in operator_dict and no_input:
            cookiecutter_dict[key] = operator_dict['default']
        else:
            # Case where no default is defined and no input - last case
            cookiecutter_dict[key] = operator_dict

        if append_key:
            return cookiecutter_dict[key]

    return cookiecutter_dict



# -*- coding: utf-8 -*-

"""Functions for generating a project from a project template."""
from __future__ import unicode_literals
from __future__ import print_function

from PyInquirer import prompt

from cookiecutter.operators import BaseOperator


class InquirerCheckboxOperator(BaseOperator):
    """Operator for PyInquirer type prompts."""

    type = 'checkbox'

    def __init__(self, operator_dict, context=None):
        """Initialize PyInquirer Hook."""  # noqa
        super(InquirerCheckboxOperator, self).__init__(
            operator_dict=operator_dict, context=context
        )

    def execute(self):
        """Run the prompt."""  # noqa
        if 'name' not in self.operator_dict:
            self.operator_dict.update({'name': 'tmp'})
            return prompt([self.operator_dict])['tmp']
        else:
            return prompt([self.operator_dict])



import cookiecutter
from cookiecutter.operators import BaseOperator


class CookiecutterOperator(BaseOperator):
    """Operator for PyInquirer type prompts."""

    type = 'cookiecutter'

    def __init__(self, operator_dict, context=None):
        """Initialize PyInquirer Hook."""  # noqa
        super(CookiecutterOperator, self).__init__(
            operator_dict=operator_dict, context=context
        )

    def execute(self):
        """Run the prompt."""  # noqa

        if 'checkout' in self.operator_dict:
            checkout = self.operator_dict['checkout']
        else:
            checkout = None

        result = cookiecutter.main.cookiecutter(
            template=self.operator_dict['template'], checkout=checkout,
        )

        return result



import sys
from cookiecutter.operators import BaseOperator
import subprocess


class CommandOperator(BaseOperator):
    """Operator for PyInquirer type prompts."""

    type = 'command'

    def __init__(self, operator_dict, context=None):
        """Initialize PyInquirer Hook."""  # noqa
        super(CommandOperator, self).__init__(
            operator_dict=operator_dict, context=context
        )

    def execute(self):
        """Run the prompt."""  # noqa
        p = subprocess.Popen(
            self.operator_dict['command'],
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        output, err = p.communicate()

        if err:
            sys.exit(err)
        return output.decode("utf-8")
